@@ -1,12 +1,47 @@
 from .ffmpeg_reader import FFmpegReader, FFmpegReaderNV
 from .ffmpeg_writer import FFmpegWriter, FFmpegWriterNV
+from .video_info import get_num_NVIDIA_GPUs
+import shutil
+from subprocess import DEVNULL, check_output
+
+
+def _check():
+    if not shutil.which('ffmpeg'):
+        raise RuntimeError('ffmpeg is not installed')
+    if not shutil.which('ffprobe'):
+        raise RuntimeError('ffprobe is not installed')
+
+_check()
+
+_check_nvidia_init = None
+
+def _check_nvidia():
+    global _check_nvidia_init
+    run = lambda x: check_output(x, shell=True, stderr=DEVNULL)
+    if _check_nvidia_init is None:
+        calling_output = run('ffmpeg -h encoder=hevc_nvenc')
+        if 'AVOptions' not in calling_output.decode('utf-8'):
+            raise RuntimeError('The ffmpeg is not compiled with NVENC support')
+
+        calling_output = run('ffmpeg -h decoder=hevc_cuvid')
+        if 'AVOptions' not in calling_output.decode('utf-8'):
+            raise RuntimeError('The ffmpeg is not compiled with NVDEC support')
+
+        if get_num_NVIDIA_GPUs() == 0:
+            raise RuntimeError('No NVIDIA GPU found')
+
+        _check_nvidia_init = True
+
+    return True
 
 
 def VideoCapture(file, 
                  codec=None, 
                  pix_fmt='bgr24',
+                 crop_xywh=None,
                  resize=None,
-                 resize_keepratio=True):
+                 resize_keepratio=True,
+                 resize_keepratioalign='center'):
     """
     Alternative to cv2.VideoCapture
 
@@ -18,10 +53,15 @@ def VideoCapture(file,
         Codec to use. Optional. Default is `None`.
     pix_fmt : str
         Pixel format. ['bgr24' | 'rgb24']. Optional. Default is 'bgr24'.
+    crop_xywh : tuple
+        Crop the frame. (x, y, width, height). Optional. Default is `None`.
     resize  : tuple
         Resize the video to the given size. Optional. Default is `None`.
     resize_keepratio : bool
         Keep the aspect ratio and the border is black. Optional. Default is `True`.
+    resize_keepratioalign : str
+        Align the image to the `center`, `topleft`, `topright`, `bottomleft` or
+        `bottomright`. Optional. Default is 'center'.
 
     Examples
     --------
@@ -64,6 +104,11 @@ def VideoCapture(file,
     cap = ffmpegcv.VideoCapture(file, pix_fmt='rgb24')
     ```
 
+    Crop video.
+    ```python
+    cap = ffmpegcv.VideoCapture(file, crop_xywh=(0, 0, 640, 480))
+    ```
+
     Resize the video to the given size
     ```
     cap = ffmpegcv.VideoCapture(file, resize=(640, 480))
@@ -73,11 +118,15 @@ def VideoCapture(file,
     ```
     cap = ffmpegcv.VideoCapture(file, resize=(640, 480), resize_keepratio=True)
     ```
-
+    
+    Crop and then resize the video.
+    ```python
+    cap = ffmpegcv.VideoCapture(file, crop_xywh=(0, 0, 640, 480), resize=(512, 512))
+    ```
     Author: Chenxinfeng 2022-04-16, cxf529125853@163.com
     """
-    print(file, codec, pix_fmt, resize, resize_keepratio)
-    return FFmpegReader.VideoReader(file, codec, pix_fmt, resize, resize_keepratio)
+    return FFmpegReader.VideoReader(file, codec, pix_fmt, crop_xywh,
+                                    resize, resize_keepratio, resize_keepratioalign)
 
 
 def VideoReader(*args, **kwargs):
@@ -156,12 +205,16 @@ def VideoCaptureNV(file,
                    pix_fmt='bgr24',
                    crop_xywh=None,
                    resize=None,
-                   resize_keepratio=None,
+                   resize_keepratio=True,
+                   resize_keepratioalign='center',
                    gpu=0):
     """
     `ffmpegcv.VideoCaptureNV` is a gpu version for `ffmpegcv.VideoCapture`.
     """
-    return FFmpegReaderNV.VideoReader(file, pix_fmt, crop_xywh, resize, resize_keepratio, gpu)
+    _check_nvidia()
+    return FFmpegReaderNV.VideoReader(file, pix_fmt, crop_xywh, 
+                                      resize, resize_keepratio, resize_keepratioalign, 
+                                      gpu)
 
 
 def VideoReaderNV(*args, **kwargs):
@@ -180,4 +233,5 @@ def VideoWriterNV(file,
     """
     `ffmpegcv.VideoWriterNV` is a gpu version for `ffmpegcv.VideoWriter`.
     """
+    _check_nvidia()
     return FFmpegWriterNV.VideoWriter(file, codec, fps, frameSize, pix_fmt, gpu)
